/* This file is part of Mailfromd.             -*- c -*-
   Copyright (C) 2007, 2008, 2009, 2010, 2011 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <mailutils/mailutils.h>

#define MF_SIEVE_LOG                0x01
#define MF_SIEVE_DEBUG_TRACE        0x02
#define MF_SIEVE_DEBUG_INSTR        0x04

MF_STATE(eom)
MF_CAPTURE(mstr)	
MF_DEFUN(sieve, NUMBER, STRING script, OPTIONAL, NUMBER dbg)
{
	mu_sieve_machine_t mach;
	int sieve_debug_flags = 0;
	int sieve_log = 0;
	int rc = mu_sieve_machine_init(&mach);
	int retval = 0;
	int f = MF_OPTVAL(dbg);
	
	MF_ASSERT(rc == 0, mfe_failure,
		  _("failed to initialize sieve machine: %s"),
		  mu_strerror(rc));

	if (f) {
		if (f & MF_SIEVE_DEBUG_TRACE)
			sieve_debug_flags |= MU_SIEVE_DEBUG_TRACE;
		if (f & MF_SIEVE_DEBUG_INSTR)
			sieve_debug_flags |= MU_SIEVE_DEBUG_INSTR;
		if (f & MF_SIEVE_LOG)
			sieve_log = 1;
	}

	mu_sieve_set_debug_level(mach, sieve_debug_flags);

	rc = mu_sieve_compile(mach, script);
	if (rc == 0) {
		mu_attribute_t attr;
		mu_message_t msg;

		rc = mu_stream_to_message(mstr, &msg);
		if (rc) {
			mu_sieve_machine_destroy(&mach);
			MF_THROW(mfe_failure,
				 _("cannot translate stream to message: %s"),
				   mu_strerror (rc));
		}
		mu_stream_unref(mstr);
		
		mu_message_get_attribute(msg, &attr);
		mu_attribute_unset_deleted(attr);
		rc = mu_sieve_message(mach, msg);
		if (rc == 0)
			retval = !(mu_attribute_is_deleted(attr) == 0);
		/* FIXME: Cannot destroy msg because this will cause
		   destroying its stream (mstr).
		   Need a workaround. */
		/*mu_message_destroy(&msg, mu_message_get_owner(msg));*/
		if (rc) {
			mu_sieve_machine_destroy(&mach);
			MF_THROW(mfe_failure,
				 _("sieving failed: %s"),
				 mu_strerror(rc));
		}
	} else {
		MF_THROW(mfe_failure,
			 _("compilation of Sieve script %s failed"),
			 script);
	}
	
	mu_sieve_machine_destroy(&mach);

	MF_RETURN(retval);
}
END

MF_INIT
